
# 🌐 **BugScanX** - Your All-in-One SNI Bug Host Finder


**BugScanX** is a cutting-edge toolkit developed by **Ayan Rajpoot** for discovering and analyzing SNI bug hosts commonly found in free internet configurations. It offers a wide array of utilities, including subdomain discovery, IP lookups, DNS record retrieval, and much more.

---

## 🚀 **Features**

BugScanX is packed with powerful tools to streamline your SNI bug hunting process:

- ⚡ **Host Checker**: A professional-grade host checker with multiple scanning modes and HTTP methods.
- 🖥️ **Subdomains Scanner**: Detect subdomains that might indicate SNI bug hosts.
- 📡 **IP Addresses Scanner**: Analyze specific IP addresses to find potential SNI bug hosts.
- 🌐 **Subdomains Finder**: Identify active subdomains associated with a target domain.
- 🔍 **Domains Hosted on Same IP**: List other domains hosted on the same IP as a given target.
- 🧰 **TXT Toolkit**: A versatile toolkit for handling text files—split, clean, remove duplicates, and more.
- 🔓 **Open Port Checker**: Scan for open ports on a given IP address or domain.
- 📜 **DNS Records**: Retrieve A, MX, TXT, and other DNS records of any domain.
- 💡 **Host OSINT**: Gather open-source intelligence (OSINT) on any host.
- 📖 **Help**: Provides detailed usage instructions for every tool.
- ⛔ **Exit**: Clean exit from the script.
- 🔄️ **Update**: Check for the latest script version and update automatically.

---

## 🎨 **UI Design**

- 🌈 **Colorful & Dynamic**: An engaging, bright interface that makes using BugScanX a fun experience.
- 😊 **Emoji-Powered**: Emojis are integrated throughout the script to bring a personal and interactive touch.
- 🚀 **Modern & Minimalist**: A clean, intuitive interface for smooth, efficient navigation.
- ⚡ **Fast & Responsive**: Optimized for high performance, providing instant feedback and a seamless experience.

---

## 📥 **Installation**

### Prerequisites
- **Python 3.x** should be installed on your machine.

### Automatic Dependency Installation
BugScanX will automatically install all required dependencies the first time it runs, including:
- **requests**
- **colorama**
- **ipaddress**
- **pyfiglet**
- **ssl**
- **beautifulsoup4**
- **dnspython**
- **multithreading**
- **loguru**

### Manual Installation (Optional)
To manually install dependencies, run:
```bash
pip install requests colorama ipaddress pyfiglet ssl beautifulsoup4 dnspython multithreading loguru
```

### Clone the Repository
```bash
git clone https://github.com/ayanrajpoot10/BugScanX.git
cd BugScanX
```

### Install via `pip`:
```bash
pip install bugscanx
```

---

## 🛠️ **Usage**

### Running the Script
```bash
python bugscanx.py
```

If installed via `pip`, run:
```bash
bugscanx
```

### **Menu**
Upon running, select a feature by entering its corresponding number:
```plaintext
 [1] ⚡ Host Checker (Pro Mode)
 [2] 🖥️ Subdomains Scanner
 [3] 📡 IP Addresses Scanner
 [4] 🌐 Subdomains Finder
 [5] 🔍 Domains Hosted on Same IP
 [6] 🧰 TXT Toolkit
 [7] 🔓 Open Port Checker
 [8] 📜 DNS Records
 [9] 💡 Host OSINT
 [10] 📖 Help
 [11] ⛔ Exit
 [0] 🔄️ Update
```

---

## 📘 **Documentation**

Comprehensive documentation can be found in the `docs/` directory for further details on how to use each feature and more advanced configurations.

---

## 🔒 **License**

This project is licensed under the **MIT License**. See the [LICENSE](LICENSE) file for more details.

---

## 🤝 **Contributing**

We welcome contributions! To contribute to BugScanX:
1. Open an issue if you find a bug or have suggestions.
2. Fork the repo and submit a pull request.

---

## 📞 **Support**

For any support or questions, feel free to reach out via our [Telegram channel](https://t.me/BugScanX).

---

Enjoy using **BugScanX** for all your SNI bug host discovery and network analysis needs! 🚀

---

